/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: SiroccoShell_Launcher.java 403 2011-10-03 15:22:43Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.remoteclientshell;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.StringTokenizer;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.junit.Assert;
import org.ow2.carol.jndi.spi.MultiOrbInitialContextFactory;
import org.ow2.sirocco.cloudmanager.remoteclientshell.command.SiroccoShell_BackSlashN_Command;
import org.ow2.sirocco.cloudmanager.remoteclientshell.command.SiroccoShell_ExampleCommand;
import org.ow2.sirocco.cloudmanager.remoteclientshell.command.SiroccoShell_OVFImporterTestWithJUnitCommand;
import org.ow2.sirocco.cloudmanager.remoteclientshell.command.SiroccoShell_ReservationTestWithJUnitCommand;
import org.ow2.sirocco.cloudmanager.remoteclientshell.command.SiroccoShell_SystemInstanceManager_CleanUpDestroyedSystemInstances_TestWithJUnitCommand;
import org.ow2.sirocco.cloudmanager.remoteclientshell.command.SiroccoShell_SystemInstanceManager_CreateSystemInstance_TestWithJUnitCommand;
import org.ow2.sirocco.cloudmanager.remoteclientshell.command.SiroccoShell_SystemInstanceManager_DestroySystemInstance_TestWithJUnitCommand;
import org.ow2.sirocco.cloudmanager.remoteclientshell.command.SiroccoShell_SystemInstanceManager_ListSystemInstances_TestWithJUnitCommand;
import org.ow2.sirocco.cloudmanager.remoteclientshell.command.SiroccoShell_SystemInstanceManager_StartSystemInstance_TestWithJUnitCommand;
import org.ow2.sirocco.cloudmanager.remoteclientshell.command.SiroccoShell_SystemInstanceManager_StopSystemInstance_TestWithJUnitCommand;
import org.ow2.sirocco.cloudmanager.remoteclientshell.command.SiroccoShell_SystemTemplateManager_CleanUpDestroyedSystemTemplates_TestWithJUnitCommand;
import org.ow2.sirocco.cloudmanager.remoteclientshell.command.SiroccoShell_SystemTemplateManager_DestroySystemTemplate_TestWithJUnitCommand;
import org.ow2.sirocco.cloudmanager.remoteclientshell.command.SiroccoShell_SystemTemplateManager_ListSystemTemplates_TestWithJUnitCommand;

public class SiroccoShell_Launcher {

    /**
     * Empty default constructor for utility class.
     */
    private SiroccoShell_Launcher() {

    }

    private static String prompt = SiroccoShell_Launcher.class.getSimpleName() + ">>";

    private static final String INITIAL_CONTEXT_FACTORY = MultiOrbInitialContextFactory.class.getName();

    /**
     * @param args
     * @throws NamingException
     */
    public static void main(final String[] args) throws NamingException {

        // Initialization of the shell's commands.
        ArrayList<ISiroccoShell_Command> testShell_commandsObjects_aL = new ArrayList<ISiroccoShell_Command>();

        // Add extra shell commands here.

        testShell_commandsObjects_aL.add(new SiroccoShell_BackSlashN_Command());

        // testShell_commandsObjects_aL
        // .add(new SiroccoShell_AllTheTestsWithJUnitCommand());
        testShell_commandsObjects_aL.add(new SiroccoShell_ReservationTestWithJUnitCommand());

        testShell_commandsObjects_aL.add(new SiroccoShell_BackSlashN_Command());

        // testShell_commandsObjects_aL
        // .add(new SiroccoShell_VMImageManagerTestWithJUnitCommand());
        testShell_commandsObjects_aL.add(new SiroccoShell_OVFImporterTestWithJUnitCommand());

        testShell_commandsObjects_aL.add(new SiroccoShell_BackSlashN_Command());

        testShell_commandsObjects_aL.add(new SiroccoShell_SystemInstanceManager_ListSystemInstances_TestWithJUnitCommand());
        testShell_commandsObjects_aL.add(new SiroccoShell_SystemInstanceManager_CreateSystemInstance_TestWithJUnitCommand());
        testShell_commandsObjects_aL.add(new SiroccoShell_SystemInstanceManager_StartSystemInstance_TestWithJUnitCommand());
        testShell_commandsObjects_aL.add(new SiroccoShell_SystemInstanceManager_StopSystemInstance_TestWithJUnitCommand());
        testShell_commandsObjects_aL.add(new SiroccoShell_SystemInstanceManager_DestroySystemInstance_TestWithJUnitCommand());
        testShell_commandsObjects_aL
            .add(new SiroccoShell_SystemInstanceManager_CleanUpDestroyedSystemInstances_TestWithJUnitCommand());

        testShell_commandsObjects_aL.add(new SiroccoShell_BackSlashN_Command());

        testShell_commandsObjects_aL.add(new SiroccoShell_SystemTemplateManager_ListSystemTemplates_TestWithJUnitCommand());
        testShell_commandsObjects_aL.add(new SiroccoShell_SystemTemplateManager_DestroySystemTemplate_TestWithJUnitCommand());
        testShell_commandsObjects_aL
            .add(new SiroccoShell_SystemTemplateManager_CleanUpDestroyedSystemTemplates_TestWithJUnitCommand());

        testShell_commandsObjects_aL.add(new SiroccoShell_BackSlashN_Command());

        testShell_commandsObjects_aL.add(new SiroccoShell_ExampleCommand());

        testShell_commandsObjects_aL.add(new SiroccoShell_BackSlashN_Command());

        System.out.println("\nEnter your choice amongst the following commands names (case isn't sensitive):");

        // Let's display the commands' names.
        Iterator<ISiroccoShell_Command> testShell_commandObject_temp_ITERATOR = testShell_commandsObjects_aL.iterator();
        while (testShell_commandObject_temp_ITERATOR.hasNext()) {
            ISiroccoShell_Command testShell_commandObject_temp = testShell_commandObject_temp_ITERATOR.next();
            System.out.println(testShell_commandObject_temp.getCommandName());
        }

        System.out.print("\n" + SiroccoShell_Launcher.prompt);
        BufferedReader bR = new BufferedReader(new InputStreamReader(System.in));
        String commandLine;
        try {
            commandLine = bR.readLine();
        } catch (IOException ex) {
            System.err.println("Can't read input. " + SiroccoShell_Launcher.class.getSimpleName() + " has been stopped.");
            return;
        }

        commandLine = commandLine.trim();

        if (commandLine.length() == 0) {
            System.err.println("The given input is empty. " + SiroccoShell_Launcher.class.getSimpleName()
                + " has been stopped.");
        } else {
            boolean commandFound = false;
            testShell_commandObject_temp_ITERATOR = testShell_commandsObjects_aL.iterator();
            while ((!commandFound) && (testShell_commandObject_temp_ITERATOR.hasNext())) {
                ISiroccoShell_Command testShell_commandObject_temp = testShell_commandObject_temp_ITERATOR.next();
                StringTokenizer sT = new StringTokenizer(commandLine, " ");
                if (testShell_commandObject_temp.getCommandName().equalsIgnoreCase(sT.nextToken())) {
                    commandFound = true;
                    // Here, the command's name has already been skipped
                    // from
                    // sT.
                    String commandLineWithoutCommandName = "";
                    while (sT.hasMoreElements()) {
                        commandLineWithoutCommandName = commandLineWithoutCommandName + sT.nextToken();
                        if (sT.hasMoreElements()) {
                            commandLineWithoutCommandName = commandLineWithoutCommandName + " ";
                        } else {
                            // nothing to do.
                        }
                    }

                    testShell_commandObject_temp.run(commandLineWithoutCommandName);
                } else {
                    // nothing to do.
                }
            }
            if (commandFound) {
                System.out.println("Command: \"" + commandLine + "\" has been executed. "
                    + SiroccoShell_Launcher.class.getSimpleName() + " has been stopped.");
            } else {
                System.err.println("The given command is unknown. " + SiroccoShell_Launcher.class.getSimpleName()
                    + " has been stopped.");
            }
        }
    }

    public static Object connectToRemoteEJB3(final String ejbName) throws NamingException {
        String carolPortString = "1099"; // System.getProperty("carol.port");
        Assert.assertNotNull("carol.port not set!", carolPortString);
        int carolPort = Integer.parseInt(carolPortString);

        Hashtable<String, Object> env = new Hashtable<String, Object>();
        env.put(Context.INITIAL_CONTEXT_FACTORY, SiroccoShell_Launcher.INITIAL_CONTEXT_FACTORY);
        env.put(Context.PROVIDER_URL, "rmi://localhost:" + carolPort);
        Context context = new InitialContext(env);
        return context.lookup(ejbName);
    }
}
