/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: OVFImporterTest.java 292 2011-08-31 11:42:01Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.testviaremoteclient;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.logging.Logger;

import javax.naming.NamingException;
import javax.xml.bind.JAXBException;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.ow2.sirocco.cloudmanager.admin.exception.DuplicateNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidArgumentException;
import org.ow2.sirocco.cloudmanager.admin.exception.OVFImporterException;
import org.ow2.sirocco.cloudmanager.admin.exception.PermissionDeniedException;
import org.ow2.sirocco.cloudmanager.admin.exception.VMImageWithTheGivenURLAlreadyExistsException;
import org.ow2.sirocco.cloudmanager.remoteclientshell.SiroccoShell_Launcher;
import org.ow2.sirocco.cloudmanager.service.IRemoteOVFImporter;
import org.ow2.sirocco.vmm.api.VMMException;

public class OVFImporterTest extends TestCase {

    private static IRemoteOVFImporter oVFImporter;

    /**
     * UserName for tests purpose.
     */
    private String userName;

    private static Logger log = Logger.getLogger(OVFImporterTest.class.getName());

    @Override
    protected void setUp() {
        log.info("setUp()");
        // log.info("Enters in: setUp()");

        // Let's initialize the EJB references.
        try {
            oVFImporter = (IRemoteOVFImporter) SiroccoShell_Launcher.connectToRemoteEJB3("OVFImporterBean");
        } catch (NamingException e) {
            e.printStackTrace();
            Assert.fail();
        }

        userName = "testuser";

        // There is nothing to backup.

        // log.info("Leaves: setUp()");
    }

    public void testOVFApplianceFileImportation_and_OVFApplianceParsingAndSystemTemplateCreation() {
        log.info("testOVFApplianceFileImportation_and_OVFApplianceParsingAndSystemTemplateCreation().");
        String fileLocationOnTheRemoteServer = null;
        try {
            String oVFApplianceLocation = "http://forge.ow2.org/dbimage.php?id=42";
            fileLocationOnTheRemoteServer = oVFImporter.oVFApplianceFileImportation(userName, oVFApplianceLocation);
            log.info("fileLocationOnTheRemoteServer: " + fileLocationOnTheRemoteServer);
            assertNotNull(fileLocationOnTheRemoteServer);
        } catch (OVFImporterException e) {
            e.printStackTrace();
            Assert.fail();
        } catch (MalformedURLException e) {
            e.printStackTrace();
            Assert.fail();
        } catch (IOException e) {
            e.printStackTrace();
            Assert.fail();
        } catch (InvalidArgumentException e) {
            e.printStackTrace();
            Assert.fail();
        } catch (PermissionDeniedException e) {
            e.printStackTrace();
            Assert.fail();
        }

        try {
            oVFImporter.oVFApplianceParsingAndSystemTemplateCreation(userName, fileLocationOnTheRemoteServer);
        } catch (OVFImporterException e) {
            e.printStackTrace();
            Assert.fail();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            Assert.fail();
        } catch (VMImageWithTheGivenURLAlreadyExistsException e) {
            e.printStackTrace();
            Assert.fail();
        } catch (JAXBException e) {
            e.printStackTrace();
            Assert.fail();
        } catch (VMMException e) {
            e.printStackTrace();
            Assert.fail();
        } catch (InvalidArgumentException e) {
            e.printStackTrace();
            Assert.fail();
        } catch (DuplicateNameException e) {
            e.printStackTrace();
            Assert.fail();
        } catch (PermissionDeniedException e) {
            e.printStackTrace();
            Assert.fail();
        }

        log.info("Test on: oVFImporter.... has been passed.");
    }

    @Override
    protected void tearDown() {
        log.info("tearDown()\n");
        // log.info("Enters in: tearDown()");

        // There is nothing to restore.
        oVFImporter = null;

        // log.info("Leaves: tearDown()\n");
    }
}