/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: ReservationTest.java 265 2011-08-18 08:01:55Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.testviaremoteclient;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.logging.Logger;

import javax.naming.NamingException;

import junit.framework.TestCase;

import org.junit.Assert;
import org.ow2.sirocco.cloudmanager.admin.exception.CloudAdminException;
import org.ow2.sirocco.cloudmanager.admin.exception.DuplicateNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InsufficientResourceException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidArgumentException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidNameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidPowerStateException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidProjectIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidUsernameException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVMImageException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVMSizeException;
import org.ow2.sirocco.cloudmanager.admin.exception.InvalidVirtualMachineIdException;
import org.ow2.sirocco.cloudmanager.admin.exception.NoRelevantResourcesPoolException;
import org.ow2.sirocco.cloudmanager.admin.exception.PermissionDeniedException;
import org.ow2.sirocco.cloudmanager.admin.exception.ResourceQuotaExceededException;
import org.ow2.sirocco.cloudmanager.admin.exception.UserAlreadyExistsException;
import org.ow2.sirocco.cloudmanager.admin.vo.VirtualMachineSpec;
import org.ow2.sirocco.cloudmanager.remoteclientshell.SiroccoShell_Launcher;
import org.ow2.sirocco.cloudmanager.service.IRemoteCloudAdmin;
import org.ow2.sirocco.cloudmanager.service.IRemoteUserManager;
import org.ow2.sirocco.cloudmanager.service.reservation.api.IRemoteResourcesReservation;
import org.ow2.sirocco.cloudmanager.service.reservation.api.ReservationException;
import org.ow2.sirocco.cloudmanager.service.reservation.api.TimeUnitGranularityType;

public class ReservationTest extends TestCase {

    private static IRemoteCloudAdmin cloudAdmin;

    private static IRemoteResourcesReservation resourcesReservation;

    private static IRemoteUserManager userManager;

    /**
     * UserName for tests purpose.
     */
    private String userName;

    private Integer bookingPeriodExpressedInTimeUnit_BeforeTest;

    private static Logger log = Logger.getLogger(ReservationTest.class.getName());

    @Override
    protected void setUp() throws NamingException, UserAlreadyExistsException, NoRelevantResourcesPoolException,
        InvalidArgumentException {
        log.info("setUp()");
        // log.info("Enters in: setUp()");

        // Let's initialize the EJB references.
        cloudAdmin = (IRemoteCloudAdmin) SiroccoShell_Launcher.connectToRemoteEJB3("CloudAdminBean");
        resourcesReservation = (IRemoteResourcesReservation) SiroccoShell_Launcher
            .connectToRemoteEJB3("ResourcesReservationBean");
        userManager = (IRemoteUserManager) SiroccoShell_Launcher.connectToRemoteEJB3("UserManagerBean");

        userName = "testuser";

        // Optionally Creation of testuser
        // List<UserVO> userVO_list = userManager.getAllUsersVo();
        // if (userVO_list == null) {
        // // nothing to do.
        // } else {
        // Iterator<UserVO> userVO_temp_ITERATOR = userVO_list.iterator();
        // while (userVO_temp_ITERATOR.hasNext()) {
        // UserVO userVO_temp = userVO_temp_ITERATOR.next();
        // System.out.println("userVO_temp: "
        // + userVO_temp.getUsername());
        // }
        // }

        if (userManager.getUserByUsername(userName) != null) {
            // userManager.deleteUser(username);
            // User testuser = userManager.createUser(username, "testuser",
            // "testuser@mail.com", "testuserFirstName",
            // "testuserLastName");
        } else {
            // User testuser =
            userManager.createUser(userName, "testuser", "testuser@mail.com", "testuserFirstName", "testuserLastName");
        }

        // Optionally Creation of testproject

        // Backup current bookingPeriod configuration.
        // log.info("Backup current bookingPeriod configuration: "
        // + resourcesReservation.getBookingPeriodExpressedInTimeUnit()
        // + ".");
        bookingPeriodExpressedInTimeUnit_BeforeTest = resourcesReservation.getBookingPeriodExpressedInTimeUnit();
        // log.info("Backup done.");

        // log.info("Leaves: setUp()");
    }

    public void testBookingPeriodConfiguration() throws InvalidArgumentException, NoRelevantResourcesPoolException {
        log.info("testBookingPeriodConfiguration().");
        // log.info("Enters in: testBookingPeriodConfiguration().");

        // Begining of the tests.
        resourcesReservation.configureBookingPeriod(0);
        assertEquals(0, resourcesReservation.getBookingPeriodExpressedInTimeUnit().intValue());

        log.info("Test on: iResourcesReservation.configureBookingPeriod(0) has been passed.");

        resourcesReservation.configureBookingPeriod(4);
        assertEquals(4, resourcesReservation.getBookingPeriodExpressedInTimeUnit().intValue());
        log.info("Test on: iResourcesReservation.configureBookingPeriod(4) has been passed.");

        try {
            resourcesReservation.configureBookingPeriod(null);
            // Here, an exception should have been raised.
            Assert.fail("Exception should have been raised.");
        } catch (InvalidArgumentException e) {
            assertTrue(true);
        }
        log.info("Test on: iResourcesReservation.configureBookingPeriod(null) has been passed.");

        try {
            resourcesReservation.configureBookingPeriod(-1);
            // Here, an exception should have been raised.
            Assert.fail("Exception should have been raised.");
        } catch (InvalidArgumentException e) {
            assertTrue(true);
        }
        log.info("Test on: iResourcesReservation.configureBookingPeriod(-1) has been passed.");

        // log.info("Leaves: testBookingPeriodConfiguration().");
    }

    public void testMonoCloudNominalUseCase() throws InvalidUsernameException, InvalidNameException, InvalidVMImageException,
        ResourceQuotaExceededException, DuplicateNameException, InvalidVMSizeException, InvalidProjectIdException,
        PermissionDeniedException, ReservationException, InvalidArgumentException, NoRelevantResourcesPoolException,
        InsufficientResourceException, InvalidVirtualMachineIdException, InvalidPowerStateException, CloudAdminException,
        InterruptedException {
        log.info("testMonoCloudNominalUseCase(" + userName + ").");

        Calendar todayIsTheStartTime = new GregorianCalendar();
        Calendar endTime = null;

        if (TimeUnitGranularityType.HOUR.equals(resourcesReservation.getTimeUnitGranularityType())) {
            endTime = new GregorianCalendar();
            endTime.setTimeInMillis(todayIsTheStartTime.getTimeInMillis() + 2 * 60 * 60 * 1000);
        } else if (TimeUnitGranularityType.DAY.equals(resourcesReservation.getTimeUnitGranularityType())) {
            endTime = new GregorianCalendar();
            endTime.setTimeInMillis(todayIsTheStartTime.getTimeInMillis() + 2 * 24 * 60 * 60 * 1000);
        } else if (TimeUnitGranularityType.WEEK.equals(resourcesReservation.getTimeUnitGranularityType())) {
            endTime = new GregorianCalendar();
            endTime.setTimeInMillis(todayIsTheStartTime.getTimeInMillis() + 2 * 7 * 24 * 60 * 60 * 1000);
        }

        // XXX projectId = username;
        String projectId = userName;

        Integer numberOfCPUCore = 5;
        Integer memorySizeInMB = 4096;
        Integer storageSizeInMB = 10240;

        if (endTime != null) {

            System.out.println("Reserve resources.");

            int numberOfResaBefore = resourcesReservation.listResourcesReservations().size();

            Integer reservationId = resourcesReservation.reserveResources(projectId, todayIsTheStartTime, endTime,
                numberOfCPUCore, memorySizeInMB, storageSizeInMB, "dummy", "Grenoble");

            int numberOfResaAfter = resourcesReservation.listResourcesReservations().size();

            assertEquals(numberOfResaBefore + 1, numberOfResaAfter);

            int numberOfVMBefore;
            int numberOfVMAfter;

            //

            System.out.println("Create VM on reserved resources.");

            numberOfVMBefore = cloudAdmin.getAllVirtualMachinesVo().size();

            VirtualMachineSpec spec = new VirtualMachineSpec();
            spec.setVmName("testvirtualmachine");
            spec.setSizeName("small");
            spec.setImageId(2);
            spec.setUserData(new HashMap<String, String>());
            spec.setReservationId(new Integer(reservationId));

            // There is no need to specify the location, indeed, it is
            // already specified in the reservation.
            boolean startVM = true;
            String vMIdOfTheVMToBeDestroyed = cloudAdmin.createAndStartVirtualMachine(projectId, userName, spec, startVM);

            numberOfVMAfter = cloudAdmin.getAllVirtualMachinesVo().size();

            assertEquals(numberOfVMBefore + 1, numberOfVMAfter);

            //

            System.out.println("Create VM num 2 on reserved resources.");

            numberOfVMBefore = cloudAdmin.getAllVirtualMachinesVo().size();

            spec.setVmName("testvirtualmachine_2");

            cloudAdmin.createAndStartVirtualMachine(projectId, userName, spec, startVM);

            numberOfVMAfter = cloudAdmin.getAllVirtualMachinesVo().size();

            assertEquals(numberOfVMBefore + 1, numberOfVMAfter);

            int sleep = 20000;
            System.out.println("Let's sleep " + sleep + " milliseconds.");
            Thread.sleep(sleep);

            //

            System.out.println("Destroy first created VM.");

            numberOfVMBefore = cloudAdmin.getAllVirtualMachinesVo().size();
            cloudAdmin.destroyVirtualMachine(vMIdOfTheVMToBeDestroyed, userName);

            numberOfVMAfter = cloudAdmin.getAllVirtualMachinesVo().size();

            assertEquals(numberOfVMBefore - 1, numberOfVMAfter);

            //

            System.out.println("Cancel reservation.");

            numberOfResaBefore = resourcesReservation.listResourcesReservations().size();
            numberOfVMBefore = cloudAdmin.getAllVirtualMachinesVo().size();

            resourcesReservation.cancelAResourcesReservation(userName, reservationId);

            numberOfResaAfter = resourcesReservation.listResourcesReservations().size();
            numberOfVMAfter = cloudAdmin.getAllVirtualMachinesVo().size();

            // Warning, deleted reservations aren't returned by
            // resourcesReservation.listResourcesReservations().
            assertEquals(numberOfResaBefore - 1, numberOfResaAfter);

            assertEquals(numberOfVMBefore - 1, numberOfVMAfter);

            sleep = 10000;
            System.out.println("Let's sleep " + sleep + " milliseconds.");
            Thread.sleep(sleep);

            System.out.println("deleteAReservationFromDataBase(" + reservationId + ").");

            assertTrue(resourcesReservation.deleteAReservationFromDataBase(reservationId));

            System.out.println("End.");

        } else {
            // Here, endTime is null.
            Assert.fail("EndTime is null.");
        }

        // System.out
        // .println("Leaves: ResourcesReservationTestModuleImpl.test_nominalUseCase("
        // + userName + ").");
    }

    public void testMultiCloudNominalUseCase() throws InvalidUsernameException, InvalidNameException, InvalidVMImageException,
        ResourceQuotaExceededException, DuplicateNameException, InvalidVMSizeException, InvalidProjectIdException,
        PermissionDeniedException, ReservationException, InterruptedException, InvalidArgumentException,
        NoRelevantResourcesPoolException, InsufficientResourceException, InvalidPowerStateException,
        InvalidVirtualMachineIdException, CloudAdminException {
        log.info("testMultiCloudNominalUseCase(" + userName + ").");

        Calendar todayIsTheStartTime = new GregorianCalendar();
        Calendar endTime = null;

        if (TimeUnitGranularityType.HOUR.equals(resourcesReservation.getTimeUnitGranularityType())) {
            endTime = new GregorianCalendar();
            endTime.setTimeInMillis(todayIsTheStartTime.getTimeInMillis() + 2 * 60 * 60 * 1000);
        } else if (TimeUnitGranularityType.DAY.equals(resourcesReservation.getTimeUnitGranularityType())) {
            endTime = new GregorianCalendar();
            endTime.setTimeInMillis(todayIsTheStartTime.getTimeInMillis() + 2 * 24 * 60 * 60 * 1000);
        } else if (TimeUnitGranularityType.WEEK.equals(resourcesReservation.getTimeUnitGranularityType())) {
            endTime = new GregorianCalendar();
            endTime.setTimeInMillis(todayIsTheStartTime.getTimeInMillis() + 2 * 7 * 24 * 60 * 60 * 1000);
        }

        // XXX projectId = username;
        String projectId = userName;

        Integer numberOfCPUCore = 5;
        Integer memorySizeInMB = 4096;
        Integer storageSizeInMB = 10240;

        if (endTime != null) {
            System.out.println("Reserve resources.");

            int numberOfResaBefore = resourcesReservation.listResourcesReservations().size();

            Integer reservationId = resourcesReservation.reserveResources(projectId, todayIsTheStartTime, endTime,
                numberOfCPUCore, memorySizeInMB, storageSizeInMB, "dummy", "Grenoble");

            int numberOfResaAfter = resourcesReservation.listResourcesReservations().size();

            assertEquals(numberOfResaBefore + 1, numberOfResaAfter);

            int numberOfResaBefore_Paris = resourcesReservation.listResourcesReservations().size();

            Integer reservationId_Paris = resourcesReservation.reserveResources(projectId, todayIsTheStartTime, endTime,
                numberOfCPUCore, memorySizeInMB, storageSizeInMB, "dummy", "Paris");

            int numberOfResaAfter_Paris = resourcesReservation.listResourcesReservations().size();

            assertEquals(numberOfResaBefore_Paris + 1, numberOfResaAfter_Paris);

            int numberOfVMBefore;
            int numberOfVMAfter;
            int numberOfVMBefore_Paris;
            int numberOfVMAfter_Paris;

            //

            System.out.println("Create VM on reserved resources.");

            numberOfVMBefore = cloudAdmin.getAllVirtualMachinesVo().size();

            VirtualMachineSpec spec = new VirtualMachineSpec();
            spec.setVmName("testvirtualmachine");
            spec.setSizeName("small");
            spec.setImageId(2);
            spec.setUserData(new HashMap<String, String>());
            spec.setReservationId(new Integer(reservationId));

            // There is no need to specify the location, indeed, it is
            // already specified in the reservation.
            boolean startVM = true;
            String vMIdOfTheVMToBeDestroyed = cloudAdmin.createAndStartVirtualMachine(projectId, userName, spec, startVM);

            numberOfVMAfter = cloudAdmin.getAllVirtualMachinesVo().size();

            assertEquals(numberOfVMBefore + 1, numberOfVMAfter);

            //

            System.out.println("Create VM on reserved resources (Paris).");

            numberOfVMBefore_Paris = cloudAdmin.getAllVirtualMachinesVo().size();

            VirtualMachineSpec spec_Paris = new VirtualMachineSpec();
            spec_Paris.setVmName("testvirtualmachine_Paris");
            spec_Paris.setSizeName("small");
            spec_Paris.setImageId(1);
            spec_Paris.setUserData(new HashMap<String, String>());
            spec_Paris.setReservationId(new Integer(reservationId_Paris));

            // There is no need to specify the location, indeed, it is
            // already specified in the reservation.

            // String vMIdOfTheVMToBeDestroyed_Paris =
            cloudAdmin.createAndStartVirtualMachine(projectId, userName, spec_Paris, startVM);

            numberOfVMAfter_Paris = cloudAdmin.getAllVirtualMachinesVo().size();

            assertEquals(numberOfVMBefore_Paris + 1, numberOfVMAfter_Paris);

            //

            System.out.println("Create VM num 2 on reserved resources.");

            numberOfVMBefore = cloudAdmin.getAllVirtualMachinesVo().size();

            spec.setVmName("testvirtualmachine_2");

            cloudAdmin.createAndStartVirtualMachine(projectId, userName, spec, startVM);

            numberOfVMAfter = cloudAdmin.getAllVirtualMachinesVo().size();

            assertEquals(numberOfVMBefore + 1, numberOfVMAfter);

            int sleep = 20000;
            System.out.println("Let's sleep " + sleep + " milliseconds.");
            Thread.sleep(sleep);

            //

            System.out.println("Destroy first created VM.");

            numberOfVMBefore = cloudAdmin.getAllVirtualMachinesVo().size();
            cloudAdmin.destroyVirtualMachine(vMIdOfTheVMToBeDestroyed, userName);

            numberOfVMAfter = cloudAdmin.getAllVirtualMachinesVo().size();

            assertEquals(numberOfVMBefore - 1, numberOfVMAfter);

            //

            System.out.println("Cancel reservation.");

            numberOfResaBefore = resourcesReservation.listResourcesReservations().size();
            numberOfVMBefore = cloudAdmin.getAllVirtualMachinesVo().size();

            resourcesReservation.cancelAResourcesReservation(userName, reservationId);

            numberOfResaAfter = resourcesReservation.listResourcesReservations().size();
            numberOfVMAfter = cloudAdmin.getAllVirtualMachinesVo().size();

            // Deleted reservations aren't returned by
            // resourcesReservation.listResourcesReservations().
            assertEquals(numberOfResaBefore - 1, numberOfResaAfter);

            assertEquals(numberOfVMBefore - 1, numberOfVMAfter);

            numberOfResaBefore_Paris = resourcesReservation.listResourcesReservations().size();
            numberOfVMBefore_Paris = cloudAdmin.getAllVirtualMachinesVo().size();

            resourcesReservation.cancelAResourcesReservation(userName, reservationId_Paris);

            numberOfResaAfter_Paris = resourcesReservation.listResourcesReservations().size();
            numberOfVMAfter_Paris = cloudAdmin.getAllVirtualMachinesVo().size();

            // Deleted reservations aren't returned by
            // resourcesReservation.listResourcesReservations().
            assertEquals(numberOfResaBefore_Paris - 1, numberOfResaAfter_Paris);

            assertEquals(numberOfVMBefore_Paris - 1, numberOfVMAfter_Paris);

            System.out.println("deleteAReservationFromDataBase(" + reservationId + ").");

            assertTrue(resourcesReservation.deleteAReservationFromDataBase(reservationId));

            System.out.println("deleteAReservationFromDataBase(" + reservationId_Paris + ").");

            assertTrue(resourcesReservation.deleteAReservationFromDataBase(reservationId_Paris));

            System.out.println("End.");

        } else {
            // Here, endTime is null.
            Assert.fail("EndTime is null.");
        }

        // System.out
        // .println("Leaves: ResourcesReservationTestModuleImpl.test_nominalMultiCloudUseCase("
        // + userName + ").");

    }

    @Override
    protected void tearDown() {
        log.info("tearDown()\n");
        // log.info("Enters in: tearDown()");

        // Restore the saved bookingPeriod configuration.
        try {
            // log.info("Restore the saved bookingPeriod configuration: "
            // + bookingPeriodExpressedInTimeUnit_BeforeTest + ".");
            resourcesReservation.configureBookingPeriod(bookingPeriodExpressedInTimeUnit_BeforeTest);
            // log.info("Restoration done.");
        } catch (InvalidArgumentException e) {
            log.severe("Restoration failed.");
            e.printStackTrace();
        } catch (NoRelevantResourcesPoolException e) {
            log.severe("Restoration failed.");
            e.printStackTrace();
        }

        cloudAdmin = null;
        resourcesReservation = null;
        userManager = null;

        // log.info("Leaves: tearDown()\n");
    }
}