/**
 *
 * SIROCCO
 * Copyright (C) 2011 France Telecom
 * Contact: sirocco@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  $Id: VMImageManagerTest.java 292 2011-08-31 11:42:01Z chazalet $
 *
 */

package org.ow2.sirocco.cloudmanager.testviaremoteclient;

import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import javax.naming.NamingException;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.ow2.sirocco.cloudmanager.admin.exception.InvalidArgumentException;
import org.ow2.sirocco.cloudmanager.admin.exception.NoRelevantResourcesPoolException;
import org.ow2.sirocco.cloudmanager.admin.exception.UserAlreadyExistsException;
import org.ow2.sirocco.cloudmanager.admin.exception.VMImageWithTheGivenURLAlreadyExistsException;
import org.ow2.sirocco.cloudmanager.admin.vo.VMImageVO;
import org.ow2.sirocco.cloudmanager.remoteclientshell.SiroccoShell_Launcher;
import org.ow2.sirocco.cloudmanager.service.IRemoteVMImageManager;
import org.ow2.sirocco.vmm.api.VMMException;

public class VMImageManagerTest extends TestCase {

    private static IRemoteVMImageManager vMImageManager;

    private static Logger log = Logger.getLogger(VMImageManagerTest.class.getName());

    @Override
    protected void setUp() throws NamingException, UserAlreadyExistsException, NoRelevantResourcesPoolException,
        InvalidArgumentException {
        log.info("setUp()");
        // log.info("Enters in: setUp()");

        // Let's initialize the EJB references.
        vMImageManager = (IRemoteVMImageManager) SiroccoShell_Launcher.connectToRemoteEJB3("VMImageManagerBean");

        // There is nothing to backup.

        // log.info("Leaves: setUp()");
    }

    public void testUploadVMImage() throws InvalidArgumentException, NoRelevantResourcesPoolException {
        log.info("testUploadVMImage().");
        try {
            int numberOfVMImages_BeforeUpload = vMImageManager.getVMImageFiles().size();
            log.info("numberOfVMImages_BeforeUpload: " + numberOfVMImages_BeforeUpload);
            vMImageManager.uploadVirtualMachineImageFile("http://vmimagestore/" + System.currentTimeMillis() + ".vmimage",
                "dummy", new Long(System.currentTimeMillis()).toString(), null, null, 1000, null, Boolean.FALSE);
            int numberOfVMImages_AfterUpload = vMImageManager.getVMImageFiles().size();
            log.info("numberOfVMImages_AfterUpload: " + numberOfVMImages_AfterUpload);
            assertEquals(numberOfVMImages_BeforeUpload + 1, numberOfVMImages_AfterUpload);
        } catch (VMMException e) {
            e.printStackTrace();
            Assert.fail();
        } catch (VMImageWithTheGivenURLAlreadyExistsException e) {
            e.printStackTrace();
            Assert.fail();
        }

        // try {
        // vMImageManager.uploadVirtualMachineImageFile(null, null, null, null,
        // null, null);
        // Assert.fail();
        // } catch (VMMException e) {
        // e.printStackTrace();
        // assertTrue(Boolean.TRUE);
        // }

        log.info("Test on: vMImageManager.uploadVirtualMachineImageFile(...) has been passed.");
    }

    public void testGetVMImageFiles() throws InvalidArgumentException, NoRelevantResourcesPoolException {
        log.info("testGetVMImageFiles().");
        List<VMImageVO> vMImageVO_list = vMImageManager.getVMImageFiles();
        int numberOfVMImages = vMImageVO_list.size();
        log.info("numberOfVMImages: " + numberOfVMImages);

        Iterator<VMImageVO> vMImageVO_temp_ITERATOR = vMImageVO_list.iterator();
        while (vMImageVO_temp_ITERATOR.hasNext()) {
            VMImageVO vMImageVO_temp = vMImageVO_temp_ITERATOR.next();
            log.info("vMImageVO_temp: " + vMImageVO_temp.getId() + ", " + vMImageVO_temp.getName() + ".");
        }
        // XXX assertEquals(numberOfVMImages_BeforeUpload + 1,
        // numberOfVMImages_AfterUpload);
        log.info("Test on: vMImageManager.getVMImageFiles() has been passed.");
    }

    @Override
    protected void tearDown() {
        log.info("tearDown()\n");
        // log.info("Enters in: tearDown()");

        // There is nothing to restore.
        vMImageManager = null;

        // log.info("Leaves: tearDown()\n");
    }
}